<?php

/*************************** LOAD THE BASE CLASS *******************************
 *******************************************************************************
 * The WP_List_Table class isn't automatically available to plugins, so we need
 * to check if it's available and load it if necessary. In this tutorial, we are
 * going to use the WP_List_Table class directly from WordPress core.
 *
 * IMPORTANT:
 * Please note that the WP_List_Table class technically isn't an official API,
 * and it could change at some point in the distant future. Should that happen,
 * I will update this plugin with the most current techniques for your reference
 * immediately.
 *
 * If you are really worried about future compatibility, you can make a copy of
 * the WP_List_Table class (file path is shown just below) to use and distribute
 * with your plugins. If you do that, just remember to change the name of the
 * class to avoid conflicts with core.
 *
 * Since I will be keeping this tutorial up-to-date for the foreseeable future,
 * I am going to work with the copy of the class provided in WordPress core.
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}


class Auctions_For_Woocommerce_Winners_List extends WP_List_Table {

	/**
	 * REQUIRED. Set up a constructor that references the parent constructor. We
	 * use the parent reference to set some default configs.
	 */
	public function __construct() {
		parent::__construct(
			array(
				'singular' => 'ref',     // singular name of the listed records
				'plural'   => 'refs',    // plural name of the listed records
				'ajax'     => false,      // does this table support ajax?
			)
		);
	}
	public function column_default( $item, $column_name ) {

		$product = wc_get_product($item['ID']);
		switch ( $column_name ) {
			case 'auction_id':
				return $item['ID'];
			case 'sku':
				return $product->get_sku();
			case 'auction_name':
				return '<a href="' . get_permalink( $item['ID'] ) . '">' . get_the_title( $item['ID'] ) . '</a>';
			case 'total_bids':
				return $product->get_auction_bids_count();
			case 'total_biders':
				return $product->get_auction_biders_count();
			case 'bid':
				return $product->get_price_html();
			case 'date':
				return $product->get_auction_dates_to();
			case 'userid':
				$userdata = get_userdata( $product->get_auction_current_bider() );
				if ( $userdata ) {
					$filter = esc_attr(add_query_arg( 'userid', $product->get_auction_current_bider() ));
					return '<a href="' . get_edit_user_link($product->get_auction_current_bider() ) . '">' . esc_attr( $userdata->user_nicename ) . '</a> <a href="' . $filter . '" class="user-filter"></a>'
						;
				} else {
					return '';
				}
			case 'name':
				$userdata = get_userdata( $product->get_auction_current_bider() );
				if ( $userdata ) {
					return  esc_attr( $userdata->user_firstname ) . ' ' . esc_attr( $userdata->user_lastname );
				} else {
					return 'n/a';
				}
			default:
				$value = isset($item[$column_name]) ? $item[$column_name] : false;
				return apply_filters( 'auctions_for_woocommerce_activity_column_default', $value, $item, $column_name );
		}
	}
	public function column_title( $item ) {
		// Return the title contents
		return sprintf(
			'%1$s <span style="color:silver">(id:%2$s)</span>%3$s',
			/*$1%s*/ $item['hits'],
			/*$2%s*/ $item['ID'],
			/*$3%s*/ $this->row_actions()
		);
	}

	public function single_row( $item ) {
		$class = apply_filters( 'auctions_for_woocommerce_activity_row_class', '', $item );
		echo '<tr class="' . esc_attr( $class ) . '">';
		$this->single_row_columns( $item );
		echo '</tr>';
	}




	/** ************************************************************************
	 * REQUIRED! This method dictates the table's columns and titles. This should
	 * return an array where the key is the column slug (and class) and the value
	 * is the column's title text. If you need a checkbox for bulk actions, refer
	 * to the $columns array below.
	 * The 'cb' column is treated differently than the rest. If including a checkbox
	 * column in your table you must create a column_cb() method. If you don't need
	 * bulk actions or checkboxes, simply leave the 'cb' entry out of your array.
	 *
	 * @see WP_List_Table::::single_row_columns()
	 * @return array An associative array containing column information: 'slugs'=>'Visible Titles'
	 **************************************************************************/
	public function get_columns() {
		$columns = array(
			'auction_id'   => esc_html__( 'Product ID', 'auctions-for-woocommerce' ),
			'sku'          => esc_html__( 'Sku', 'auctions-for-woocommerce' ),
			'auction_name' => esc_html__( 'Auction Name', 'auctions-for-woocommerce' ),
			'total_bids'   => esc_html__( 'Bids', 'auctions-for-woocommerce' ),
			'total_biders' => esc_html__( 'Bidders', 'auctions-for-woocommerce' ),
			'userid'       => esc_html__( 'Winner', 'auctions-for-woocommerce' ),
			'name'         => esc_html__( 'Name', 'auctions-for-woocommerce' ),
			'bid'          => esc_html__( 'Winning Bid', 'auctions-for-woocommerce' ),
			'date'         => esc_html__( 'Date Ended', 'auctions-for-woocommerce' ),
		);
		return apply_filters( 'auctions_for_woocommerce_activity_columns', $columns );
	}


	/** ************************************************************************
	 * Optional. If you want one or more columns to be sortable (ASC/DESC toggle),
	 * you will need to register it here. This should return an array where the
	 * key is the column that needs to be sortable, and the value is db column to
	 * sort by. Often, the key and value will be the same, but this is not always
	 * the case (as the value is a column name from the database, not the list table).
	 * This method merely defines which columns should be sortable and makes them
	 * clickable - it does not handle the actual sorting. You still need to detect
	 * the ORDERBY and ORDER querystring variables within prepare_items() and sort
	 * your data accordingly (usually by modifying your query).
	 *
	 * @return array An associative array containing all the columns that should be sortable: 'slugs'=>array('data_values',bool)
	 **************************************************************************/
	public function get_sortable_columns() {
		$sortable_columns = array(
			'date'       => array( 'end_date', false ),
		);
		return apply_filters( 'auctions_for_woocommerce_activity_sortable_columns', $sortable_columns );
	}

	/** ************************************************************************
	 * REQUIRED! This is where you prepare your data for display. This method will
	 * usually be used to query the database, sort and filter the data, and generally
	 * get it ready to be displayed. At a minimum, we should set $this->items and
	 * $this->set_pagination_args(), although the following properties and methods
	 * are frequently interacted with here...
	 *
	 * @global WPDB $wpdb
	 * @uses $this->_column_headers
	 * @uses $this->items
	 * @uses $this->get_columns()
	 * @uses $this->get_sortable_columns()
	 * @uses $this->get_pagenum()
	 * @uses $this->set_pagination_args()
	 **************************************************************************/
	public function prepare_items() {
		global $wpdb;

		$screen = get_current_screen();

		$searchstring             = isset( $_GET['s'] ) ? esc_sql( sanitize_text_field( $_GET['s'] ) ) : false;
		$date_from                = isset( $_GET['datefrom'] ) ? esc_sql( sanitize_text_field( $_GET['datefrom'] ) ) : gmdate( 'Y-m-d H:i', strtotime( '-1 year' ) );
		$date_to                  = isset( $_GET['dateto'] ) ? esc_sql( sanitize_text_field( $_GET['dateto'] ) ) : gmdate( 'Y-m-d H:i' );
		$date_from                = gmdate( 'Y-m-d H:i', strtotime( $date_from ) );
		$date_to                  = gmdate( 'Y-m-d H:i', strtotime( $date_to ) );
		$auction_visibility_terms = wc_get_auction_visibility_term_ids();
		$product_types            = wp_list_pluck(	get_terms( array( 'taxonomy'   => 'product_type', 'hide_empty' => false, ) ), 'term_taxonomy_id', 'name' );


		/* -- Ordering parameters -- */
		$orderby = ! empty( $_GET['orderby'] ) ? esc_sql( sanitize_text_field( $_GET['orderby'] ) ) : 'end_date';
		$order   = ! empty( $_GET['order'] ) ? esc_sql( sanitize_text_field( $_GET['order'] ) ) : 'DESC';

		/* -- Pagination parameters -- */
		if ( ! empty( $searchstring ) ) {
			if ( null !== apply_filters( 'wpml_default_language', null ) ) {
				$totalitems = $wpdb->query(
					$wpdb->prepare(
						'SELECT  ' . $wpdb->prefix . 'posts.ID
						FROM ' . $wpdb->prefix . 'posts
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships
						ON (' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'term_relationships.object_id)
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships AS tt1
						ON (' . $wpdb->prefix . 'posts.ID = tt1.object_id)
						LEFT JOIN ' . $wpdb->prefix . 'icl_translations AS wpml
                		ON (' . $wpdb->prefix . 'posts.ID = wpml.element_id AND wpml.element_type = "post_product")
						INNER JOIN ' . $wpdb->prefix . 'postmeta
						ON ( ' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'postmeta.post_id )
						WHERE 1=1
						AND ( ' . $wpdb->prefix . ' wp_term_relationships.term_taxonomy_id IN (%d) )
						AND ( tt1.term_taxonomy_id IN (%d,%d,%d) )
						AND ( ' . $wpdb->prefix . 'postmeta.meta_key = "_auction_dates_to" )
						AND ' . $wpdb->prefix . 'posts.post_type = "product"
						AND ((' . $wpdb->prefix . 'posts.post_status = "publish" ))
						AND  ' . $wpdb->prefix . 'postmeta.meta_value BETWEEN CAST( %s AS DATETIME) AND CAST( %s AS DATETIME)
						AND ( ' . $wpdb->prefix . 'posts.post_title LIKE %s )
						AND wpml.source_language_code IS NULL
						GROUP BY ' . $wpdb->prefix . 'posts.ID
						ORDER BY %s %s
						LIMIT %d, %d',
						$product_types[ 'auction' ],
						$auction_visibility_terms['finished'],
						$auction_visibility_terms['sold'],
						$auction_visibility_terms['buy-now'],
						$date_from,
						$date_to,
						$wpdb->esc_like( $searchstring ),
						$orderby,
						$order
					)
				);

			} else {
				$totalitems = $wpdb->query(
					$wpdb->prepare(
						'SELECT  ' . $wpdb->prefix . 'posts.ID
						FROM ' . $wpdb->prefix . 'posts
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships
						ON (' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'term_relationships.object_id)
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships AS tt1
						ON (' . $wpdb->prefix . 'posts.ID = tt1.object_id)
						INNER JOIN ' . $wpdb->prefix . 'postmeta
						ON ( ' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'postmeta.post_id )
						WHERE 1=1
						AND ( ' . $wpdb->prefix . 'term_relationships.term_taxonomy_id IN (%d) )
						AND ( tt1.term_taxonomy_id IN (%d,%d,%d) )
						AND ( ' . $wpdb->prefix . 'postmeta.meta_key = "_auction_dates_to" )
						AND ' . $wpdb->prefix . 'posts.post_type = "product"
						AND ((' . $wpdb->prefix . 'posts.post_status = "publish" ))
						AND  ' . $wpdb->prefix . 'postmeta.meta_value BETWEEN CAST( %s AS DATETIME) AND CAST( %s AS DATETIME)
						AND ( ' . $wpdb->prefix . 'posts.post_title LIKE %s )
						GROUP BY ' . $wpdb->prefix . 'posts.ID
						ORDER BY %s %s
						LIMIT %d, %d',
						$product_types[ 'auction' ],
						$auction_visibility_terms['finished'],
						$auction_visibility_terms['sold'],
						$auction_visibility_terms['buy-now'],
						$date_from,
						$date_to,
						$wpdb->esc_like( $searchstring ),
						$orderby,
						$order
					)
				);
			}


		} else {
			if ( null !== apply_filters( 'wpml_default_language', null ) ) {
				$totalitems = $wpdb->query(
					$wpdb->prepare(
						'SELECT  ' . $wpdb->prefix . 'posts.ID
						FROM ' . $wpdb->prefix . 'posts
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships
						ON (' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'term_relationships.object_id)
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships AS tt1
						ON (' . $wpdb->prefix . 'posts.ID = tt1.object_id)
						LEFT JOIN ' . $wpdb->prefix . 'icl_translations AS wpml
                		ON (' . $wpdb->prefix . 'posts.ID = wpml.element_id AND wpml.element_type = "post_product")
						INNER JOIN ' . $wpdb->prefix . 'postmeta
						ON ( ' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'postmeta.post_id )
						WHERE 1=1
						AND ( ' . $wpdb->prefix . 'term_relationships.term_taxonomy_id IN (%d) )
						AND ( tt1.term_taxonomy_id IN (%d,%d,%d) )
						AND ( ' . $wpdb->prefix . 'postmeta.meta_key = "_auction_dates_to" )
						AND ' . $wpdb->prefix . 'posts.post_type = "product"
						AND ((' . $wpdb->prefix . 'posts.post_status = "publish" ))
						AND  ' . $wpdb->prefix . 'postmeta.meta_value BETWEEN CAST( %s AS DATETIME) AND CAST( %s AS DATETIME)
						AND wpml.source_language_code IS NULL
						GROUP BY ' . $wpdb->prefix . 'posts.ID
						ORDER BY %s %s',
						$product_types[ 'auction' ],
						$auction_visibility_terms['finished'],
						$auction_visibility_terms['sold'],
						$auction_visibility_terms['buy-now'],
						$date_from,
						$date_to,
						$orderby,
						$order
					)
				);

			} else {
				$totalitems = $wpdb->query(
					$wpdb->prepare(
						'SELECT  ' . $wpdb->prefix . 'posts.ID
						FROM ' . $wpdb->prefix . 'posts
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships
						ON (' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'term_relationships.object_id)
						LEFT JOIN ' . $wpdb->prefix . 'term_relationships AS tt1
						ON (' . $wpdb->prefix . 'posts.ID = tt1.object_id)
						INNER JOIN ' . $wpdb->prefix . 'postmeta
						ON ( ' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'postmeta.post_id )
						WHERE 1=1
						AND ( ' . $wpdb->prefix . 'term_relationships.term_taxonomy_id IN (%d) )
						AND ( tt1.term_taxonomy_id IN (%d,%d,%d) )
						AND ( ' . $wpdb->prefix . 'postmeta.meta_key = "_auction_dates_to" )
						AND ' . $wpdb->prefix . 'posts.post_type = "product"
						AND ((' . $wpdb->prefix . 'posts.post_status = "publish" ))
						AND  ' . $wpdb->prefix . 'postmeta.meta_value BETWEEN CAST( %s AS DATETIME) AND CAST( %s AS DATETIME)
						GROUP BY ' . $wpdb->prefix . 'posts.ID
						ORDER BY %s %s',
						$product_types[ 'auction' ],
						$auction_visibility_terms['finished'],
						$auction_visibility_terms['sold'],
						$auction_visibility_terms['buy-now'],
						$date_from,
						$date_to,
						$orderby,
						$order
					)
				);
			}


		}

		$user       = get_current_user_id();
		$screen     = get_current_screen();
		if ( $screen ) {
			$option     = $screen->get_option( 'per_page', 'option' );
		}
		$perpage = get_user_option( 'logs_per_page', $user );
		$perpage = $perpage ? $perpage : 20;

		if ( $screen && ( empty( $perpage ) || $perpage < 1 ) ) {
			$perpage = $screen->get_option( 'per_page', 'default' );
		}

		// Which page is this?
		$paged = ! empty( $_GET['paged'] ) ? esc_sql( sanitize_text_field( $_GET['paged'] ) ) : '';

		// Page Number
		if ( empty( $paged ) || ! is_numeric( $paged ) || 0 >= $paged ) {
			$paged = 1;
		}

		// How many pages do we have in total.
		$totalpages = ceil( $totalitems / $perpage );

		$offset = ( ! empty( $paged ) && ! empty( $perpage ) ) ? ( $paged - 1 ) * $perpage : 0;

		$this->set_pagination_args(
			array(
				'total_items' => $totalitems,
				'total_pages' => $totalpages,
				'per_page'    => $perpage,
			)
		);




		$columns               = $this->get_columns();
		$hidden                = array();
		$sortable              = $this->get_sortable_columns();
		$this->_column_headers = array( $columns, $hidden, $sortable );
		if ( ! empty( $searchstring ) ) {
			$this->items           = $wpdb->get_results(
				$wpdb->prepare(
					'SELECT  ' . $wpdb->prefix . 'posts.ID
					FROM ' . $wpdb->prefix . 'posts
					LEFT JOIN ' . $wpdb->prefix . 'term_relationships
					ON (' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'term_relationships.object_id)
					LEFT JOIN ' . $wpdb->prefix . 'term_relationships AS tt1
					ON (' . $wpdb->prefix . 'posts.ID = tt1.object_id)
					INNER JOIN ' . $wpdb->prefix . 'postmeta
					ON ( ' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'postmeta.post_id )
					WHERE 1=1
					AND ( ' . $wpdb->prefix . 'term_relationships.term_taxonomy_id IN (%d) )
					AND ( tt1.term_taxonomy_id IN (%d,%d,%d) )
					AND ( ' . $wpdb->prefix . 'postmeta.meta_key = "_auction_dates_to" )
					AND ' . $wpdb->prefix . 'posts.post_type = "product"
					AND ((' . $wpdb->prefix . 'posts.post_status = "publish" ))
					AND  ' . $wpdb->prefix . 'postmeta.meta_value BETWEEN CAST( %s AS DATETIME) AND CAST( %s AS DATETIME)
					AND ( ' . $wpdb->prefix . 'posts.post_title LIKE %s )
					GROUP BY ' . $wpdb->prefix . 'posts.ID
					ORDER BY %s %s
					LIMIT %d, %d',
					$product_types[ 'auction' ],
					$auction_visibility_terms['finished'],
					$auction_visibility_terms['sold'],
					$auction_visibility_terms['buy-now'],
					$date_from,
					$date_to,
					$wpdb->esc_like( $searchstring ),
					$orderby,
					$order,
					$offset,
					$perpage
				),
				ARRAY_A
			);
		} else {

			$this->items           = $wpdb->get_results(
				$wpdb->prepare(
					'SELECT  ' . $wpdb->prefix . 'posts.ID, ' . $wpdb->prefix . 'postmeta.meta_value as end_date, ' . $wpdb->prefix . 'posts.post_title, mt3.meta_value as winner
					FROM ' . $wpdb->prefix . 'posts
					LEFT JOIN ' . $wpdb->prefix . 'term_relationships
					ON (' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'term_relationships.object_id)
					LEFT JOIN ' . $wpdb->prefix . 'term_relationships AS tt1
					ON (' . $wpdb->prefix . 'posts.ID = tt1.object_id)
					INNER JOIN ' . $wpdb->prefix . 'postmeta
					ON ( ' . $wpdb->prefix . 'posts.ID = ' . $wpdb->prefix . 'postmeta.post_id )
					LEFT JOIN ' . $wpdb->prefix . 'postmeta AS mt3 ON ( ' . $wpdb->prefix . 'posts.ID = mt3.post_id AND mt3.meta_key = "_auction_current_bider" )
					WHERE 1=1
					AND ( ' . $wpdb->prefix . 'term_relationships.term_taxonomy_id IN (%d) )
					AND ( tt1.term_taxonomy_id IN (%d,%d,%d) )
					AND ( ' . $wpdb->prefix . 'postmeta.meta_key = "_auction_dates_to" )
					AND ' . $wpdb->prefix . 'posts.post_type = "product"
					AND ((' . $wpdb->prefix . 'posts.post_status = "publish" ))
					AND  ' . $wpdb->prefix . 'postmeta.meta_value BETWEEN CAST( %s AS DATETIME) AND CAST( %s AS DATETIME)
					GROUP BY ' . $wpdb->prefix . 'posts.ID
					ORDER BY %s %s
					LIMIT %d, %d',
					$product_types[ 'auction' ],
					$auction_visibility_terms['finished'],
					$auction_visibility_terms['sold'],
					$auction_visibility_terms['buy-now'],
					$date_from,
					$date_to,
					$orderby,
					$order,
					$offset,
					$perpage
				),
				ARRAY_A
			);

		}
	}


	public function datepicker() {
		$auction_dates_from = isset( $_GET['datefrom'] ) ? esc_sql( sanitize_text_field( $_GET['datefrom'] ) ) : gmdate( 'Y-m-d H:i', strtotime( '-1 year' ) );
		$auction_dates_to   = isset( $_GET['dateto'] ) ? esc_sql( sanitize_text_field( $_GET['dateto'] ) ) : gmdate( 'Y-m-d H:i' );

		$auction_dates_from = gmdate( 'Y-m-d H:i', strtotime( $auction_dates_from ) );
		$auction_dates_to = gmdate( 'Y-m-d H:i', strtotime( $auction_dates_to ) );
		echo '  <div><p class="form-field auction_activityrange">
							<label for="_auction_dates_from">' . esc_html__( 'Date range', 'auctions-for-woocommerce' ) . '</label>
							<input type="text" class="short datetimepicker" name="datefrom" id="_auction_dates_from" value="' . esc_attr( $auction_dates_from ) . '" placeholder="' . esc_html_x( 'From&hellip; YYYY-MM-DD HH:MM', 'placeholder', 'auctions-for-woocommerce' ) . '" maxlength="16" pattern="[0-9]{4}-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[01])[ ](0[0-9]|1[0-9]|2[0-4]):(0[0-9]|1[0-9]|2[0-9]|3[0-9]|4[0-9]|5[0-9])" />
							<input type="text" class="short datetimepicker" name="dateto" id="_auction_dates_to" value="' . esc_attr( $auction_dates_to ) . '" placeholder="' . esc_html_x( 'To&hellip; YYYY-MM-DD HH:MM', 'placeholder', 'auctions-for-woocommerce' ) . '" maxlength="16" pattern="[0-9]{4}-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[01])[ ](0[0-9]|1[0-9]|2[0-4]):(0[0-9]|1[0-9]|2[0-9]|3[0-9]|4[0-9]|5[0-9])" />
							<input type="submit" id="activityrange-submit" class="button" value="submit">
				</p>';
	}
}
