<?php
/**
 * WooCommerce Template Functions
 *
 * Functions used in the template files to output content - in most cases hooked in via the template actions. All functions are pluggable.
 *
 */

defined( 'ABSPATH' ) || exit;


if ( ! function_exists( 'auctions_for_woocommerce_add_to_cart' ) ) {

	/**
	 * Output the auction product add to cart area.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_add_to_cart() {
		global $product;
		
		if ( method_exists ( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'single-product/add-to-cart/auction.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_bid' ) ) {

	/**
	 * Output the bid in auction template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_bid() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'single-product/bid.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_pay' ) ) {

	/**
	 * Output the pay for auction template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_pay() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'single-product/pay.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_bid_form' ) ) {

	/**
	 * Output the bid form template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_bid_form() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'single-product/auction-bid-form.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_condition' ) ) {

	/**
	 * Output the condition template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_condition() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-condition.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_countdown' ) ) {

	/**
	 * Output the countdown template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_countdown() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-counter.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_dates' ) ) {

	/**
	 * Output the dates template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_dates() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-dates.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_max_bid' ) ) {

	/**
	 * Output the max_bid template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_max_bid() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-max-bid.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_reserve' ) ) {

	/**
	 * Output the reserve template.
	 *
	 * @subpackage	Product
	 * @return void
		 *
	 */
	function auctions_for_woocommerce_reserve() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-reserve.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_sealed' ) ) {

	/**
	 * Output the sealed template.
	 *
	 * @subpackage	Product
	 * @return void
	 *
	 */
	function auctions_for_woocommerce_sealed() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-sealed.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_ajax_conteiner_start' ) ) {

	/**
	 * Output the ajax-conteiner-start template.
	 *
	 * @subpackage	Product
	 * @return void
	 * *
	 */
	function auctions_for_woocommerce_ajax_conteiner_start() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-ajax-conteiner-start.php' );
		}
	}
}
if ( ! function_exists( 'auctions_for_woocommerce_ajax_conteiner_end' ) ) {

	/**
	 * Output the ajax-conteiner-end template.
	 *
	 * @subpackage	Product
	 * @return void
	 *
	 */
	function auctions_for_woocommerce_ajax_conteiner_end() {
		global $product;

		if ( method_exists( $product, 'get_type') && $product->get_type() === 'auction') {
			wc_get_template( 'global/auction-ajax-conteiner-end.php' );
		}
	}
}

if ( ! function_exists( 'auctions_for_woocommerce_seated_info' ) ) {
	function auctions_for_woocommerce_seated_info() {
		wc_get_template( 'single-product/seated-auction-info.php' );
	}
}

if ( ! function_exists( 'auctions_for_woocommerce_seated_status' ) ) {
	function auctions_for_woocommerce_seated_status() {
		wc_get_template( 'single-product/seated-auction-status.php' );
	}
}

if ( ! function_exists( 'auctions_for_woocommerce_seated_progressbar_template' ) ) {
	/**
	 * Load lottery product add to cart template part.
	 *
	 */
	function auctions_for_woocommerce_seated_progressbar_template () {
		wc_get_template( 'global/auction-seats-progressbar.php' );
	}
}
