<?php
/**
 * Wc auctions Shortcode
 *
 */

class Auctions_For_Woocommerce_Shortcodes extends WC_Shortcodes {

	/**
	 * Init shortcodes.
	 */
	public static function init() {

		$shortcodes = array(
			'auctions'                             => __CLASS__ . '::auctions',
			'featured_auctions'                    => __CLASS__ . '::featured_auctions',
			'recent_auctions'                      => __CLASS__ . '::recent_auctions',
			'ending_soon_auctions'                 => __CLASS__ . '::ending_soon_auctions',
			'future_auctions'                      => __CLASS__ . '::future_auctions',
			'finished_auctions'                    => __CLASS__ . '::finished_auctions',
			'past_auctions'                        => __CLASS__ . '::finished_auctions',
			'auctions_watchlist'                   => __CLASS__ . '::auctions_watchlist',
			'my_auctions_activity'                 => __CLASS__ . '::my_auctions_activity',
			'all_user_auctions'                    => __CLASS__ . '::all_user_auctions',
			'auctions_for_woocommerce_my_auctions' => __CLASS__ . '::my_auctions',
			'auctions_recent_bids'                 => __CLASS__ . '::auctions_recent_bids',
			'auctions_templates'                   => __CLASS__ . '::auctions_templates',

		);

		foreach ( $shortcodes as $shortcode => $function ) {
			add_shortcode( apply_filters( "{$shortcode}_shortcode_tag", $shortcode ), $function );
		}

	}
	/**
	 * List multiple products shortcode.
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function auctions( $atts ) {

		$atts = (array) $atts;
		$type = 'auctions';
		// Allow list product based on specific cases.
		if ( isset( $atts['won_auctions'] ) && wc_string_to_bool( $atts['won_auctions'] ) ) {
			$type = 'won_auctions';
		} elseif ( isset( $atts['auctions_winners'] ) && wc_string_to_bool( $atts['auctions_winners'] ) ) {
			$type = 'auctions_winners';
		} elseif ( isset( $atts['my_auctions'] ) && wc_string_to_bool( $atts['my_auctions'] ) ) {
			$type = 'my_auctions';
		}
		$shortcode = new Shortcode_Auctions( $atts, $type );

		return $shortcode->get_content();
	}


	/**
	 * Output auction templates
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function auctions_templates( $atts ) {

		$template = isset ( $atts['template'] ) ? $atts['template'] : false;

		if ( $template ) {

			switch ( $template ) {
				case 'add-to-cart':
					auctions_for_woocommerce_add_to_cart();
					break;
				case 'bid':
					auctions_for_woocommerce_bid();
					break;
				case 'pay':
					auctions_for_woocommerce_pay();
					break;
				case 'bid-form':
					auctions_for_woocommerce_bid_form();
					break;
				case 'condition':
					auctions_for_woocommerce_condition();
					break;
				case 'countdown':
					auctions_for_woocommerce_countdown();
					break;
				case 'dates':
					auctions_for_woocommerce_dates();
					break;
				case 'max-bid':
					auctions_for_woocommerce_max_bid();
					break;
				case 'reserve':
					auctions_for_woocommerce_reserve();
					break;
				case 'auction-sealed':
					auctions_for_woocommerce_sealed();
					break;
				case 'ajax-conteiner-start':
					auctions_for_woocommerce_ajax_conteiner_start();
					break;
				case 'ajax-conteiner-end':
					auctions_for_woocommerce_ajax_conteiner_end();
					break;
				case 'auctions_seated_progressbar_template':
					auctions_for_woocommerce_seated_progressbar_template();
					break;

				default:
					// code...
					break;
			}

		}
	}

	/**
	 * Output featured auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function featured_auctions( $atts ) {
		$atts = array_merge(
			array(
				'limit'        => '12',
				'columns'      => '4',
				'orderby'      => 'date',
				'order'        => 'DESC',
				'category'     => '',
				'cat_operator' => 'IN',
			),
			(array) $atts
		);

		$atts['visibility'] = 'featured';

		$shortcode = new Shortcode_Auctions( $atts, 'featured_auctions' );

		return $shortcode->get_content();
	}

	/**
	 * Output recent auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function recent_auctions( $atts ) {
		$atts = array_merge(
			array(
				'limit'        => '12',
				'columns'      => '4',
				'orderby'      => 'date',
				'order'        => 'DESC',
				'category'     => '',
				'cat_operator' => 'IN',
			),
			(array) $atts
		);

		$shortcode = new Shortcode_Auctions( $atts, 'recent_auctions' );

		return $shortcode->get_content();
	}

	/**
	 * Output ending soon auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function ending_soon_auctions( $atts ) {
		$atts = array_merge(
			array(
				'limit'        => '12',
				'columns'      => '4',
				'orderby'      => 'meta_value',
				'order'        => 'ASC',
				'category'     => '',
				'meta_key' => '_auction_dates_to',
				'cat_operator' => 'IN',
				'auction_status' =>'active',
				'future' =>'yes',
			),
			(array) $atts
		);


		$shortcode = new Shortcode_Auctions( $atts, 'ending_soon_auctions' );

		return $shortcode->get_content();
	}

	/**
	 * Output future auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function future_auctions( $atts ) {
		$atts = array_merge(
			array(
				'limit'              => '12',
				'columns'            => '4',
				'orderby'            => 'date',
				'order'              => 'DESC',
				'category'           => '',
				'cat_operator'       => 'IN',
				'auction_status' =>'future',
			),
			(array) $atts
		);

		$shortcode = new Shortcode_Auctions( $atts, 'future_auctions' );

		return $shortcode->get_content();
	}

	/**
	 * Output finished auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function finished_auctions( $atts ) {
		$atts = array_merge(
			array(
				'limit'              => '12',
				'columns'            => '4',
				'orderby'            => 'date',
				'order'              => 'DESC',
				'category'           => '',
				'cat_operator'       => 'IN',
				'auction_status' =>'finished',
			),
			(array) $atts
		);

		$shortcode = new Shortcode_Auctions( $atts, 'future_auctions' );

		return $shortcode->get_content();
	}

	/**
	 * Output finished auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function auctions_watchlist( $atts ) {
		if ( is_user_logged_in() ) {
			$user_ID       = get_current_user_id();
			$watchlist_ids = get_user_meta( $user_ID, '_auction_watch' );
			if ( ! empty ( $watchlist_ids ) ) {
				$atts = array_merge(
					array(
						'limit'              => '12',
						'columns'            => '4',
						'orderby'            => 'date',
						'order'              => 'DESC',
						'category'           => '',
						'cat_operator'       => 'IN',
						'ids'             =>  implode( ',' , $watchlist_ids),
					),
					(array) $atts
				);

				$shortcode = new Shortcode_Auctions( $atts, 'future_auctions' );

				return $shortcode->get_content();
			} else {
				return wc_get_template_html( 'loop/no-products-found.php' );
			}
		} else {
			return '<p class="woocommerce-info">' . esc_html__( 'Please log in to see your auction watchlist.', 'auctions-for-woocommerce' ) . '</p>';
		}

	}

	/**
	 * Output finished auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function my_auctions_activity( $atts ) {
		global $wpdb;

		if ( is_user_logged_in() ) {

			$atts = shortcode_atts(
				array(
					'limit' => 10,

				),
				$atts
			);

			$user_id = get_current_user_id();

			$useractivity = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM `' . $wpdb->prefix . 'auctions_for_woocommerce_log` WHERE `userid` = %d ORDER BY date DESC limit %d ', $user_id, $atts['limit'] ) );

			if ( $useractivity ) {
				return wc_get_template_html( 'shortcodes/my-auctions-activity.php', array( 'useractivity' => $useractivity , 'user_id' => $user_id ) );
			}
		} else {
			return  '<div class="woocommerce"><p class="woocommerce-info">' . esc_html__( 'Please log in to see your auctions activity.', 'auctions-for-woocommerce' ) . '</p></div>';
		}

	}
	 /**
		 * All_user_auctions shortcode
		 *
		 * @param array $atts
		 * @return string
		 */
	public static function all_user_auctions( $atts ) {
		global $wpdb;

		if ( is_user_logged_in() ) {

			$atts = shortcode_atts(
				array(
					'limit' => '500',

				),
				$atts
			);

			$user_id     = get_current_user_id();
			$postids     = array();
			$userauction = $wpdb->get_results( $wpdb->prepare( 'SELECT DISTINCT auction_id FROM ' . $wpdb->prefix . 'auctions_for_woocommerce_log WHERE userid = %d ORDER BY date DESC limit %d  ', $user_id, $atts['limit'] ), ARRAY_N );
			if ( isset( $userauction ) && ! empty( $userauction ) ) {
				foreach ( $userauction as $auction ) {
					$postids [] = $auction[0];

				}
			}
			$atts = array_merge(
			array(
				'limit'              => '12',
				'columns'            => '4',
				'orderby'            => 'date',
				'order'              => 'DESC',
				'category'           => '',
				'cat_operator'       => 'IN',
				'auction_arhive'     => true,
				'show_past_auctions' => true,
				'ids'             =>  implode( ',' , $postids),

			),
			(array) $atts
			);

			$shortcode = new Shortcode_Auctions( $atts, 'all_user_auctions' );

			return $shortcode->get_content();



		} else {
			return  '<div class="woocommerce"><p class="woocommerce-info">' . esc_html__( 'Please log in to see all your auctions.', 'auctions-for-woocommerce' ) . '</p></div>';
		}
	}

	/**
	 * Output my active auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function my_active_auctions( $atts ) {
		
		if ( is_user_logged_in() ) {

			$user_id = get_current_user_id();
			$postids = auctions_for_woocommerce_get_user_auctions();

			if ( !empty( $postids ) ) {
				if ( is_array( $postids ) ) {
					$postids = implode (',', $postids );
				}


				$atts = array_merge(
					array(
						'limit'              => '-1',
						'ids'                => $postids,
						'columns'            => '4',
						'orderby'            => 'date',
						'order'              => 'DESC',
						'category'           => '',
						'cat_operator'       => 'IN',
						'auction_status' =>'active',
					),
					(array) $atts
				);

				$shortcode = new Shortcode_Auctions( $atts, 'my_active_auctions' );

				return $shortcode->get_content();

			} else {
				$output = '<div class="woocommerce"><p class="woocommerce-info">' . esc_html__("You don't have any active auction.", 'auctions_for_woocommerce' ) . '</p></div>';
				return $output;
			}

		}
	}

	/**
	 * Output won auctions shortcode
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function won_auctions( $atts ) {

		if ( is_user_logged_in() ) {

			$atts = array_merge(
				array(
					'limit'              => '-1',
					'columns'            => '4',
					'orderby'            => 'meta_value',
					'order'              => 'ASC',
					'category'           => '',
					'cat_operator'       => 'IN',
					'meta_key'           => '_auction_dates_to',
					'auction_status'     =>'finished',
				),
				(array) $atts
			);

			$shortcode = new Shortcode_Auctions( $atts, 'my_won_auctions' );

			return $shortcode->get_content();
		}
	}

	/**
	 * Output my auction shortcode.
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function my_auctions( $atts ) {

		if ( is_user_logged_in() ) {
			$output = '<div class="wc-auctions active-auctions clearfix woocommerce"><h2>' . esc_html__( 'Active auction', 'auctions_for_woocommerce' ) . '</h2>';
			$output .= self::my_active_auctions($atts);
			$output .= '</div><div class="wc-auctions active-auctions clearfix woocommerce"><h2>' . esc_html__( 'Won auctions', 'auctions_for_woocommerce' ) . '</h2>';
			$output .= self::won_auctions($atts);
			$output .= '</div>';
			return $output;

		} else {
			$output = '<div class="woocommerce"><p class="woocommerce-info">' . esc_html__('Please log in to see your auctions.', 'auctions_for_woocommerce' ) . '</p></div>';
			return $output;
		}
	}

	/**
	 * Output auction winners shortcode.
	 *
	 * @param array $atts Attributes.
	 * @return string
	 */
	public static function auctions_winners( $atts ) {

				$atts = array_merge(
					array(
						'limit'              => '-1',
						'columns'            => '4',
						'orderby'            => 'meta_value',
						'order'              => 'DESC',
						'category'           => '',
						'cat_operator'       => 'IN',
						'meta_key'           => '_auction_dates_to',
						'auction_status' =>'finished',
					),
					(array) $atts
				);

				$shortcode = new Shortcode_Auctions( $atts, 'auctions_winners' );

				return $shortcode->get_content_auction_winners();



	}

	/**
	 * Auctions recent bids shortcode
	 *
	 * @param array $atts
	 * @return string
	 */
	public static function auctions_recent_bids( $atts ) {
		global $wpdb;

		$atts = shortcode_atts(
			array(
				'limit'          => 20,
				'show_usernames' => 'yes',

			),
			$atts
		);
		$user_id          = get_current_user_id();
		$auction_activity = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM ' . $wpdb->prefix . 'auctions_for_woocommerce_log ORDER BY date DESC limit %d ', $atts['limit'] ) );

		if ( $auction_activity ) {
			return wc_get_template_html( 'shortcodes/auctions-recent-bids.php', array( 'auction_activity' => $auction_activity , 'user_id' => $user_id,  'atts' => $atts  ) );

		} else {
			echo '<div class="woocommerce"><p class="woocommerce-info">' . esc_html__( 'There is not any bids at the moment!.', 'auctions-for-woocommerce' ) . '</p></div>';
		}

	}




	
}

new Auctions_For_Woocommerce_Shortcodes();
